@@ -21,9 +21,9 @@ except ImportError:
     from urllib3.response import HTTPResponse
 
 if six.PY2:
-    from urlparse import urlparse, parse_qsl
+    from urlparse import urlparse, parse_qsl, urlsplit, urlunsplit
 else:
-    from urllib.parse import urlparse, parse_qsl
+    from urllib.parse import urlparse, parse_qsl, urlsplit, urlunsplit
 
 if six.PY2:
     try:
@@ -102,12 +102,12 @@ class CallList(Sequence, Sized):
         self._calls = []
 
 
-def _ensure_url_default_path(url, match_querystring):
-    if _is_string(url) and url.count('/') == 2:
-        if match_querystring:
-            return url.replace('?', '/?', 1)
-        else:
-            return url + '/'
+def _ensure_url_default_path(url):
+    if _is_string(url):
+        url_parts = list(urlsplit(url))
+        if url_parts[2] == '':
+            url_parts[2] = '/'
+        url = urlunsplit(url_parts)
     return url
 
 
@@ -127,7 +127,7 @@ class BaseResponse(object):
         self.method = method
         self.match_querystring = match_querystring
         # ensure the url has a default path set if the url is a string
-        self.url = _ensure_url_default_path(url, match_querystring)
+        self.url = _ensure_url_default_path(url)
         self.call_count = 0
 
     def _url_matches_strict(self, url, other):
@@ -139,6 +139,10 @@ class BaseResponse(object):
 
         url_qsl = sorted(parse_qsl(url_parsed.query))
         other_qsl = sorted(parse_qsl(other_parsed.query))
+
+        if len(url_qsl) != len(other_qsl):
+            return False
+
         for (a_k, a_v), (b_k, b_v) in zip(url_qsl, other_qsl):
             if not isinstance(a_k, six.text_type):
                 a_k = a_k.decode('utf-8')
@@ -232,7 +236,7 @@ class Response(BaseResponse):
 
         return HTTPResponse(
             status=status,
-            reason=six.moves.http_client.responses[status],
+            reason=six.moves.http_client.responses.get(status),
             body=body,
             headers=headers,
             preload_content=False, )
@@ -264,7 +268,7 @@ class CallbackResponse(BaseResponse):
 
         return HTTPResponse(
             status=status,
-            reason=six.moves.http_client.responses[status],
+            reason=six.moves.http_client.responses.get(status),
             body=body,
             headers=headers,
             preload_content=False, )
@@ -382,13 +386,17 @@ class RequestsMock(object):
         return get_wrapped(func, _wrapper_template, evaldict)
 
     def _find_match(self, request):
-        for match in self._matches:
+        found = None
+        found_match = None
+        for i, match in enumerate(self._matches):
             if match.matches(request):
-                break
-        else:
-            return None
-
-        return match
+                if found is None:
+                    found = i
+                    found_match = match
+                else:
+                    # Multiple matches found.  Remove & return the first match.
+                    return self._matches.pop(found)
+        return found_match
 
     def _on_request(self, adapter, request, **kwargs):
         match = self._find_match(request)
@@ -409,6 +417,7 @@ class RequestsMock(object):
                 request,
                 match.get_response(request), )
         except Exception as response:
+            match.call_count += 1
             self._calls.add(request, response)
             response = resp_callback(response) if resp_callback else response
             raise
