@@ -94,6 +94,27 @@ def test_match_querystring():
         assert_response(resp, 'test')
         resp = requests.get('http://example.com?foo=bar&test=1')
         assert_response(resp, 'test')
+        resp = requests.get('http://example.com/?foo=bar&test=1')
+        assert_response(resp, 'test')
+
+    run()
+    assert_reset()
+
+
+def test_match_empty_querystring():
+    @responses.activate
+    def run():
+        responses.add(
+            responses.GET,
+            'http://example.com',
+            body=b'test',
+            match_querystring=True)
+        resp = requests.get('http://example.com')
+        assert_response(resp, 'test')
+        resp = requests.get('http://example.com/')
+        assert_response(resp, 'test')
+        with pytest.raises(ConnectionError):
+            requests.get('http://example.com?query=foo')
 
     run()
     assert_reset()
@@ -211,6 +232,19 @@ def test_no_content_type():
     assert_reset()
 
 
+def test_arbitrary_status_code():
+    @responses.activate
+    def run():
+        url = 'http://example.com/'
+        responses.add(responses.GET, url, body='test', status=418)
+        resp = requests.get(url)
+        assert resp.status_code == 418
+        assert resp.reason is None
+
+    run()
+    assert_reset()
+
+
 def test_throw_connection_error_explicit():
     @responses.activate
     def run():
@@ -455,6 +489,14 @@ def test_assert_all_requests_are_fired():
             requests.get('http://example.com')
             assert len(m._matches) == 1
 
+        # check that assert_all_requests_are_fired=True counts mocked errors
+        with responses.RequestsMock(assert_all_requests_are_fired=True) as m:
+            m.add(responses.GET, 'http://example.com', body=Exception())
+            assert len(m._matches) == 1
+            with pytest.raises(Exception):
+                requests.get('http://example.com')
+            assert len(m._matches) == 1
+
     run()
     assert_reset()
 
@@ -552,3 +594,36 @@ def test_legacy_adding_headers():
 
     run()
     assert_reset()
+
+
+def test_multiple_responses():
+    @responses.activate
+    def run():
+        responses.add(responses.GET, 'http://example.com', body='test')
+        responses.add(responses.GET, 'http://example.com', body='rest')
+
+        resp = requests.get('http://example.com')
+        assert_response(resp, 'test')
+        resp = requests.get('http://example.com')
+        assert_response(resp, 'rest')
+        # After all responses are used, last response should be repeated
+        resp = requests.get('http://example.com')
+        assert_response(resp, 'rest')
+
+    run()
+    assert_reset()
+
+
+def test_multiple_urls():
+    @responses.activate
+    def run():
+        responses.add(responses.GET, 'http://example.com/one', body='one')
+        responses.add(responses.GET, 'http://example.com/two', body='two')
+
+        resp = requests.get('http://example.com/two')
+        assert_response(resp, 'two')
+        resp = requests.get('http://example.com/one')
+        assert_response(resp, 'one')
+
+    run()
+    assert_reset()
